import 'dart:io';

import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xtoolspackage/xtoolspackage.dart';
// import 'package:fijkplayer/fijkplayer.dart';

class XMediaPlayer<T> extends StatefulWidget {
  //Ci facciamo passare l'oggetto da visualizzare T
  XMediaPlayer(
    Key? key, {
    required this.itemOriginal,
    required this.onTakePicture,
    required this.onStopVideoRecording,
    this.title,
    this.title_BackColor,
    this.title_Color,
  });

  String? title;
  Color? title_BackColor;
  Color? title_Color;

  T itemOriginal;
  Function(XFile, T, bool, void Function(void Function())) onTakePicture;
  Function(XFile, T, bool, void Function(void Function())) onStopVideoRecording;
  @override
  State<XMediaPlayer<T>> createState() => XMediaPlayerState<T>();
}

class XMediaPlayerState<T> extends State<XMediaPlayer<T>> with WidgetsBindingObserver, TickerProviderStateMixin {
  late CameraController? photoController;
  int pointers = 0;
  static String? urlRTSP;
  XFile? imageFile;
  XFile? videoFile;
  VideoPlayerController? videoController;
  VoidCallback? videoPlayerListener;
  double _minAvailableZoom = 1.0;
  double _maxAvailableZoom = 1.0;
  double _currentScale = 1.0;
  double _baseScale = 1.0;
  double _minAvailableExposureOffset = 0.0;
  double _maxAvailableExposureOffset = 0.0;
  // FijkPlayer player = FijkPlayer();
  bool outCamera = false;

  bool isBusy = false;
  String waiterMessage = "";

  TextEditingController? urlRTSPcontroller = TextEditingController();
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    XStatefulWidgetBase.color_Waiter = Color.fromARGB(255, 22, 228, 255);
    loadPlayer();
    if (urlRTSP == null) {
      urlRTSPcontroller!.text = "rtsp://";
    } else {
      urlRTSPcontroller!.text = urlRTSP ?? "";
    }
    super.initState();
  }

  loadPlayer() async {
    await onNewCameraSelected();
    setFlashMode(FlashMode.off, setState);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    photoController!.dispose();
    urlRTSPcontroller!.dispose();
    // player.dispose();
    super.dispose();
  }

  void showInSnackBar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text(message)));
  }

  Future<void> onNewCameraSelected() async {
    // final CameraController? oldController = photoController;
    // if (oldController != null) {
    //   photoController = null;
    //   await oldController.dispose();
    // }
    late List<CameraDescription> cameras = <CameraDescription>[]; //TODO
    photoController = CameraController(cameras[0], ResolutionPreset.medium, enableAudio: true);

    photoController!.addListener(() {
      if (mounted) {
        setState(() {});
      }
      if (photoController!.value.hasError) {
        showInSnackBar('Camera error ${photoController!.value.errorDescription}');
      }
    });

    try {
      await photoController!.initialize();
      await Future.wait(<Future<Object?>>[
        ...!kIsWeb ? <Future<Object?>>[photoController!.getMinExposureOffset().then((double value) => _minAvailableExposureOffset = value), photoController!.getMaxExposureOffset().then((double value) => _maxAvailableExposureOffset = value)] : <Future<Object?>>[],
        photoController!.getMaxZoomLevel().then((double value) => _maxAvailableZoom = value),
        photoController!.getMinZoomLevel().then((double value) => _minAvailableZoom = value),
      ]);
    } on CameraException catch (e) {
      switch (e.code) {
        case 'CameraAccessDenied':
          showInSnackBar('Non hai gli accessi alla telecamera.');
          break;
        case 'CameraAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi alla Telecamera.");
          break;
        case 'CameraAccessRestricted':
          // iOS only
          showInSnackBar('Accesso riservato per la Telecamera');
          break;
        case 'AudioAccessDenied':
          showInSnackBar("Non hai gli accessi per registrare l'audio");
          break;
        case 'AudioAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi all'Audio.");
          break;
        case 'AudioAccessRestricted':
          // iOS only
          showInSnackBar("Accesso riservato per la registrazione Audio");
          break;
        default:
          _showCameraException(e);
          break;
      }
    }

    if (mounted) {
      setState(() {});
    }
  }

  void _handleScaleStart(ScaleStartDetails details) {
    _baseScale = _currentScale;
  }

  Future<void> _handleScaleUpdate(ScaleUpdateDetails details) async {
    if (pointers != 2) {
      return;
    }
    _currentScale = (_baseScale * details.scale).clamp(_minAvailableZoom, _maxAvailableZoom);
    await photoController!.setZoomLevel(_currentScale);
  }

  Widget _cameraPreviewWidget() {
    return Listener(
        onPointerDown: (_) => pointers++,
        onPointerUp: (_) => pointers--,
        child: CameraPreview(photoController!, child: LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
          return GestureDetector(
            behavior: HitTestBehavior.opaque,
            onScaleStart: _handleScaleStart,
            onScaleUpdate: _handleScaleUpdate,
            onTapDown: (TapDownDetails details) => onViewFinderTap(details, constraints),
          );
        })));
  }

  void onViewFinderTap(TapDownDetails details, BoxConstraints constraints) {
    if (photoController! == null) {
      return;
    }

    final Offset offset = Offset(
      details.localPosition.dx / constraints.maxWidth,
      details.localPosition.dy / constraints.maxHeight,
    );
    photoController!.setExposurePoint(offset);
    photoController!.setFocusPoint(offset);
  }

  Future<XFile?> takePicture() async {
    final CameraController? cameraController = photoController!;
    if (cameraController == null || !cameraController.value.isInitialized) {
      return null;
    }
    if (cameraController.value.isTakingPicture) {
      return null;
    }
    try {
      final XFile file = await cameraController.takePicture();
      return file;
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  void _showCameraException(CameraException e) {
    print(e.description);
    showInSnackBar('Error: ${e.code}\n${e.description}');
  }

  void onTakePictureButtonPressed(T lav, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sto salvando la Foto...";
      isBusy = true;
    });
    takePicture().then((XFile? file) async {
      if (mounted) {
        this.setState(() {
          imageFile = file;
          videoController?.dispose();
          videoController = null;
        });
        if (file != null) {
          await widget.onTakePicture(file, lav, isBusy, setState);
          setState(() {
            waiterMessage = "";
            isBusy = false;
          });
          return file;
        }
      }
    });
  }

  void onVideoRecordButtonPressed(T lav, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sta per cominciare la Registrazione...";
      isBusy = true;
    });
    startVideoRecording(lav, setState).then((_) {
      if (mounted) {
        setState(() {
          waiterMessage = "";
          isBusy = false;
        });
      }
    });
  }

  Widget _captureControlRowWidget(T lav, void Function(void Function()) setState) {
    Color color_BackGroundBTNactions = Color.fromARGB(255, 0, 45, 97);
    return Container(
        margin: EdgeInsets.only(right: 5),
        child: Column(mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: <Widget>[
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: Icon(Icons.flash_on_outlined),
                color: photoController!.value.flashMode == FlashMode.off
                    ? Colors.white
                    : photoController!.value.isInitialized
                        ? Colors.yellow
                        : Colors.white,
                onPressed: photoController!.value.isInitialized
                    ? photoController!.value.flashMode == FlashMode.off
                        ? () => setFlashMode(FlashMode.torch, setState)
                        : () => setFlashMode(FlashMode.off, setState)
                    : null,
              )),
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: photoController!.value.isRecordingPaused ? const Icon(Icons.play_arrow) : const Icon(Icons.pause),
                color: Colors.white,
                onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo
                    ? (photoController!.value.isRecordingPaused)
                        ? () => onResumeButtonPressed(setState)
                        : () => onPauseButtonPressed(setState)
                    : null,
              )),
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: Icon(Icons.stop),
                color: Colors.red,
                onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo ? () => onStopButtonPressed(lav, setState) : null,
              )),
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: Icon(Icons.camera_alt),
                color: Colors.white,
                onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => onTakePictureButtonPressed(lav, setState) : null,
              )),
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: Icon(Icons.videocam),
                color: Colors.white,
                onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => onVideoRecordButtonPressed(lav, setState) : null,
              )),
          Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
              child: IconButton(
                icon: Icon(Icons.video_camera_back_outlined),
                color: outCamera ? Colors.green : Colors.white,
                onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => cmdViewAnotherCamera() : null,
              )),
        ]));
  }

  cmdViewAnotherCamera() async {
    if (outCamera.not()) {
      var x = await showDialog(
          context: context,
          builder: (context) {
            return XAlertDialog(
              title_Text: "Inserisci URL rtsp:",
              child: Column(mainAxisSize: MainAxisSize.min, children: [
                Row(
                  children: [
                    Expanded(
                        child: XTextFormField(
                      controller: urlRTSPcontroller,
                      height: null,
                    ))
                  ],
                )
              ]),
              actionsBTNarea_Child: Row(children: [
                Expanded(
                    child: XBtnbase(
                  margin: EdgeInsets.symmetric(horizontal: 10),
                  label: "Annulla",
                  onPressed: () {
                    urlRTSP = "";
                    Navigator.pop(context, false);
                  },
                  label_Color: Colors.red,
                )),
                Expanded(
                    child: XBtnbase(
                  margin: EdgeInsets.symmetric(horizontal: 10),
                  label: "Salva",
                  onPressed: () {
                    if (urlRTSPcontroller!.text != urlRTSP) {
                      urlRTSP = urlRTSPcontroller!.text;
                    } else {}
                    outCamera = outCamera.not();
                    Navigator.pop(context, true);
                  },
                  label_Color: Colors.green,
                ))
              ]),
            );
          });
      setState(() {
        isBusy = true;
      });
      if (x != null && x) {
        // player.setOption(FijkOption.hostCategory, "enable-snapshot", 1);
        // player.setOption(FijkOption.playerCategory, "mediacodec-all-videos", 1);
        // player = FijkPlayer();
        if (urlRTSP != null) {
          // await player.setDataSource(urlRTSP!, autoPlay: true); //'rtsp://admin:ldmospiti@192.168.1.120:554/live/CAM2'
        }
        // player.isPlayable();
      } else {
        outCamera = false;
      }

      setState(() {
        isBusy = false;
      });
    } else {
      setState(() {
        outCamera = outCamera.not();
      });
    }
  }

  Future<void> _startVideoPlayer(T lav) async {
    final VideoPlayerController vController = kIsWeb ? VideoPlayerController.network(videoFile!.path) : VideoPlayerController.file(File(videoFile!.path));

    videoPlayerListener = () {
      if (videoController != null && videoController!.value.size != null) {
        if (mounted) {
          setState(() {});
        }
        videoController!.removeListener(videoPlayerListener!);
      }
    };
    vController.addListener(videoPlayerListener!);
    await vController.setLooping(true);
    await vController.initialize();
    await videoController?.dispose();
    if (mounted) {
      setState(() {
        imageFile = null;
        videoController = vController;
      });
    }
    await vController.play();
  }

  void onStopButtonPressed(T lav, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sto salvando la Registrazione...";
      isBusy = true;
    });
    stopVideoRecording(lav, setState).then((XFile? file) {
      if (mounted) {
        setState(() {});
      }
      if (file != null) {
        videoFile = file;
      }
    });
  }

  Future<void> onPausePreviewButtonPressed(void Function(void Function()) setState) async {
    final CameraController? cameraController = photoController!;

    if (cameraController == null || !cameraController.value.isInitialized) {
      // showInSnackBar('Error: select a camera first.');
      return;
    }
    if (cameraController.value.isPreviewPaused) {
      await cameraController.resumePreview();
    } else {
      await cameraController.pausePreview();
    }

    if (mounted) {
      setState(() {});
    }
  }

  void onPauseButtonPressed(void Function(void Function()) setState) {
    pauseVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video messo in Pausa');
    });
  }

  void onResumeButtonPressed(void Function(void Function()) setState) {
    resumeVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video ripartito');
    });
  }

  Future<void> startVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isInitialized) {
      showInSnackBar('Error: Prima seleziona un telecamera.');
      return;
    }

    if (photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.startVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return;
    }
  }

  Future<XFile?> stopVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return null;
    }

    try {
      var y = await photoController!.stopVideoRecording();
      videoFile = y;
      widget.onStopVideoRecording(videoFile!, lav, isBusy, setState);
      setState(() {
        waiterMessage = "";
        isBusy = false;
      });
      return videoFile;
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  Future<void> pauseVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.pauseVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<void> resumeVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.resumeVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<bool> setFlashMode(FlashMode mode, void Function(void Function()) setState) async {
    if (photoController! == null) {
      return false;
    }

    try {
      await photoController!.setFlashMode(mode);
      if (photoController!.value.flashMode == FlashMode.off) {
        setState(() {});
        return false;
      } else {
        setState(() {});
        return true;
      }
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  // Widget _thumbnailWidget() {
  //   final VideoPlayerController? localVideoController = videoController;

  //   return Expanded(
  //       child: Align(
  //           alignment: Alignment.centerRight,
  //           child: Row(mainAxisSize: MainAxisSize.min, children: <Widget>[
  //             if (localVideoController == null && imageFile == null)
  //               Container()
  //             else
  //               SizedBox(
  //                   width: 64.0,
  //                   height: 64.0,
  //                   child: (localVideoController == null)
  //                       ? (kIsWeb ? Image.network(imageFile!.path) : Image.file(File(imageFile!.path)))
  //                       : Container(
  //                           decoration: BoxDecoration(border: Border.all(color: Colors.pink)),
  //                           child: Center(
  //                             child: AspectRatio(aspectRatio: localVideoController.value.size != null ? localVideoController.value.aspectRatio : 1.0, child: VideoPlayer(localVideoController)),
  //                           )))
  //           ])));
  // }

  @override
  Widget build(BuildContext context) {
    return (photoController != null && photoController!.value.isInitialized)
        ? Container(
            width: MediaQuery.of(context).size.width - 1,
            height: MediaQuery.of(context).size.height - 1,
            child: Padding(
                padding: EdgeInsets.all(1.0),
                child: Center(
                    child: Column(children: <Widget>[
                  Expanded(
                      child: Container(
                    decoration: BoxDecoration(color: Colors.black, border: Border.all(width: 3.0, color: photoController!.value.isRecordingVideo ? Colors.redAccent : Colors.grey)),
                    child: Padding(
                        padding: EdgeInsets.all(1.0),
                        child: Center(
                            child: Stack(
                          alignment: Alignment.centerRight,
                          children: [
                            _cameraPreviewWidget(),
                            // outCamera == false
                            //     ? _cameraPreviewWidget()
                            //     : FijkView(
                            //         player: player,
                            //),
                            _captureControlRowWidget(widget.itemOriginal, setState),
                            isBusy ? Container(color: Colors.black.withAlpha(90)) : Container(),
                            isBusy ? XStatefulWidgetBase.LoaderIndicator(isBusy, label: waiterMessage) : Container(),
                          ],
                        ))),
                  )),
                  // _modeControlRowWidget(),
                  // Padding(
                  //   padding: const EdgeInsets.all(5.0),
                  //   child: Row(
                  //     children: <Widget>[
                  //       _cameraTogglesRowWidget(),
                  //       _thumbnailWidget(),
                  //     ],
                  //   ),
                  // ),
                ]))))
        : XStatefulWidgetBase.LoaderIndicator(true);
  }
}
